@@ -19,7 +19,8 @@ Efficient seeking within sorted text files. Works by implementing in-place
 binary search on the lines of the file, with a small hack to handle the first
 line of the file.
 
-See accompanying README.md for more information.
+See accompanying documentation for more information.
+http://sortedfile.readthedocs.org/
 """
 
 # If user specifies the exact start of a line using the `lo` parameter, due to
@@ -38,7 +39,8 @@ except ImportError:
 
 
 def getsize(fp):
-    """Return the size of `fp` if possible, otherwise raise ValueError."""
+    """Return the size of `fp` if it is a physical file, ``StringIO``, or
+    ``mmap.mmap``, otherwise raise ValueError."""
     if hasattr(fp, 'getvalue'):
         return len(fp.getvalue())
     elif _mmap and isinstance(fp, _mmap):
@@ -49,6 +51,18 @@ def getsize(fp):
         raise ValueError("can't get size of %r" % (fp,))
 
 
+def warm(fp, lo=None, hi=None):
+    """Encourage the seekable file `fp` to become cached by reading from it
+    sequentially."""
+    lo = lo or 0
+    hi = hi or getsize(fp)
+    fp.seek(lo)
+    s = ' '
+    while s and hi >= 0:
+        s = fp.read(10485760)
+        hi -= len(s) if s else hi
+
+
 def bisect_seek_left(fp, x, lo=None, hi=None, key=None):
     """Position the sorted seekable file `fp` such that all preceding lines are
     less than `x`. If `x` is present, the file is positioned on its first
@@ -139,8 +153,65 @@ def bisect_seek_fixed_right(fp, n, x, lo=None, hi=None, key=None):
     fp.seek(lo + (rlo * n))
 
 
+def bisect_func_left(x, lo, hi, func):
+    """Bisect `func(i)`, returning an index such that preceding values are less
+    than `x`. If `x` is present, the returned index is its first occurrence.
+    EOF is assumed if `func` returns None."""
+    while lo < hi:
+        mid = (lo + hi) // 2
+        k = func(mid)
+        if k is not None and k < x:
+            lo = mid + 1
+        else:
+            hi = mid
+
+    return lo
+
+
+def bisect_func_right(x, lo, hi, func):
+    """Bisect `func(i)`, returning an index such that consecutive values are
+    greater than `x`. If `x` is present, the returned index is past its last
+    occurrence. EOF is assumed if `func` returns None."""
+    while lo < hi:
+        mid = (lo + hi) // 2
+        k = key_at(mid)
+        if k is not None and x < k:
+            hi = mid
+        else:
+            lo = mid + 1
+
+    return lo
+
+
+def extents(fp, lo=None, hi=None):
+    """Return a tuple of the first and last lines from the seekable file
+    `fp`."""
+    lo = (lo - 1) if lo else 0
+    hi = hi or getsize(fp)
+    bisect_seek_left(fp, '', lo, hi)
+    low = fp.readline()
+
+    for offset in xrange(0, 1048576, 4096):
+        fp.seek(hi - offset)
+        _, sep, high = fp.read(offset - 1).rstrip('\n').rpartition('\n')
+        if sep:
+            return low, high
+
+
+def extents_fixed(fp, n, lo=None, hi=None):
+    """Return a tuple of the first and last `n` byte records from the seekable
+    file `fp`."""
+    lo = lo or 0
+    hi = hi or getsize(fp)
+    bisect_seek_fixed_left(fp, n, '', lo, hi)
+    low = fp.read(n)
+    recs = (hi - lo) // n
+    fp.seek(lo + (n * (recs - 1)))
+    return low, fp.read(n)
+
+
 def iter_inclusive(fp, x, y, lo=None, hi=None, key=None):
-    """Iterate lines of the sorted seekable file `fp` satisfying the condition
+    """Iterate lines of the sorted seekable file `fp` satisfying
     `x <= line <= y`."""
     key = key or (lambda s: s)
     bisect_seek_left(fp, x, lo, hi, key)
@@ -149,7 +220,7 @@ def iter_inclusive(fp, x, y, lo=None, hi=None, key=None):
 
 
 def iter_exclusive(fp, x, y, lo=None, hi=None, key=None):
-    """Iterate lines of the sorted seekable file `fp` satisfying the condition
+    """Iterate lines of the sorted seekable file `fp` satisfying
     `x < line < y`."""
     key = key or (lambda s: s)
     bisect_seek_right(fp, x, lo, hi, key)
@@ -158,8 +229,8 @@ def iter_exclusive(fp, x, y, lo=None, hi=None, key=None):
 
 
 def iter_fixed_inclusive(fp, n, x, y, lo=None, hi=None, key=None):
-    """Iterate `n` byte records of the sorted seekable file `fp` satisfying the
-    condition `x <= record <= y`."""
+    """Iterate `n` byte records of the sorted seekable file `fp` satisfying
+    `x <= record <= y`."""
     key = key or (lambda s: s)
     bisect_seek_fixed_left(fp, n, x, lo, hi, key)
     pred = lambda s: x <= key(s) <= y
@@ -167,8 +238,8 @@ def iter_fixed_inclusive(fp, n, x, y, lo=None, hi=None, key=None):
 
 
 def iter_fixed_exclusive(fp, n, x, y, lo=None, hi=None, key=None):
-    """Iterate `n` byte records of the sorted seekable file `fp` satisfying the
-    condition `x < record < y`."""
+    """Iterate `n` byte records of the sorted seekable file `fp` satisfying
+    `x < record < y`."""
     key = key or (lambda s: s)
     bisect_seek_fixed_right(fp, n, x, lo, hi, key)
     pred = lambda s: x < key(s) < y
